import os, cPickle, stat
from random import shuffle
from Globals import DTMLFile
from zLOG import LOG, INFO
from AccessControl import ClassSecurityInfo
from Products.PageTemplates.PageTemplateFile import PageTemplateFile

from Constants import *

from slimmer import slimmer, acceptableSyntax


def _generateIdentifier(basepath, relpath, mtime):
    id = basepath + relpath
    id = id.replace('\\','').replace('/','').replace(' ','')
    id = id.replace('.','').replace(':','')
    id = list(id[:15])
    shuffle(id)
    id = "".join(id)+str(mtime)[-4:]
    return id


def _getAllconfigs():
    items = []
    if os.path.isfile(CONFIGFILEPATH):
        items.extend(cPickle.load(open(CONFIGFILEPATH, 'r')))
            
    return items


def _getconfigIdentifier(basepath, relpath):
    """ Check if there alread is one called this """
    items = []
    items.extend(_getAllconfigs())
    for item in items:
        if item['basepath']==basepath and item['relpath']==relpath:
            return item
    return None



def _delPickleconfig(identifier):
    items = []
    items.extend(_getAllconfigs())
    
    checked_items = []
    for item in items:
        if item['identifier'] != identifier:
            checked_items.append(item)

    cPickle.dump(checked_items, open(CONFIGFILEPATH, 'w'))



def _write2Pickleconfig(basepath, relpath, description, mtime,
                        filetype='DTML', checkfirst=1):
    """ save an item to pickle file """

    # add this new one
    addthis = 1
    if checkfirst:
        item = _getconfigIdentifier(basepath, relpath)
        if item is not None:
            addthis = 0
            if mtime > item['mtime']: # has changed
                _delPickleconfig(item['identifier'])
                addthis = 1
		
    items=[]
    items.extend(_getAllconfigs())

    if addthis:
        identifier = _generateIdentifier(basepath, relpath, mtime)
        d={'description':description, 'filetype':filetype,
           'relpath':relpath, 'basepath':basepath,
           'mtime':mtime, 'identifier':identifier}
        items.append(d)

        # save new file
        cPickle.dump(items, open(CONFIGFILEPATH, 'w'))

        return 1
    else:
        return 0
    
def write2config(basepath, relpath, description, mtime,
                 filetype='DTML', checkfirst=1):
    """ save a config item """
    return _write2Pickleconfig(basepath, relpath, description, mtime,
                               filetype=filetype, checkfirst=checkfirst)
    

    
    
class CTDTMLFile(DTMLFile):
    def __init__(self, name, _prefix=None, **kw):
        " doc string "
        
        description = kw.get('description',kw.get('d',''))
	
        # 'optimize' argument
        optimize = kw.get('optimize', kw.get('opt', None))
        self.optimize = acceptableSyntax(optimize)
            
        if not kw.has_key('uncheckoutable'):
            sep = name[max(name.find('\\'), name.find('/'))]
            self.ctnamesplit = name.split(sep)

            prodpath = _prefix['__name__'].split('.')[:-1]
            prodpath = ".".join(prodpath).replace('.',os.sep)

            basepath = os.path.join(CT_INSTANCE_HOME, prodpath)
            if not os.path.isdir(basepath):
                basepath = os.path.join(CT_SOFTWARE_HOME, prodpath)

            
            # first, open file to see how old it is.
            name = name.replace('/', os.sep).replace('\\',os.sep)
            
            t = os.stat(os.path.join(basepath, name+ '.dtml'))
            mtime = t[stat.ST_MTIME]
            
            # if already there there's no point to
            # add it again
            added = write2config(basepath, name, description,
                                 mtime, 'DTML')

            if DEBUG and added:
                LOG(self.__class__.__name__, INFO, name)

        apply(DTMLFile.__init__, (self, name, _prefix), kw)


    def _exec(self, bound_data, args, kw):
        """ Execute the template but first look for one instanciated
        in the ZODB based on the some filename conversion. """

        ctnamesplit = bound_data['self'].ctnamesplit
        if len(self.ctnamesplit) == 1:
            possibletemplate = ctnamesplit[0] + '.dtml'
        else:
            possibletemplate = '.'.join(ctnamesplit[1:]) + '.dtml'
        base = getattr(bound_data['context'], 'aq_base',
                       bound_data['context'])
        

        if hasattr(base, possibletemplate):
            request = bound_data['context'].REQUEST
            params = (bound_data['context'], request)
            result = apply(base[possibletemplate], params)
        else:
            result = apply(DTMLFile._exec, (self, bound_data, args, kw))
	    
        if self.optimize and OPTIMIZE:
            result = slimmer(result, self.optimize)
	    
        return result
    



class CTPageTemplateFile(PageTemplateFile):
    """ CTPageTemplateFile subclasses PageTemplateFile and
    intercepts the __init__ and the _exec methods """
    
    def __init__(self, name, _prefix=None, **kw):
        """ doc string """
        
        description = kw.get('description',kw.get('d',''))

	    # 'optimize' argument
        optimize = kw.get('optimize', kw.get('opt', None))
        self.optimize = acceptableSyntax(optimize)
	
        if not kw.has_key('uncheckoutable'):
            sep = name[max(name.find('\\'), name.find('/'))]
            self.ctnamesplit = name.split(sep)
            
            prodpath = _prefix['__name__'].split('.')[:-1]
            prodpath = ".".join(prodpath).replace('.',os.sep)
            
            basepath = os.path.join(CT_INSTANCE_HOME, prodpath)
            if not os.path.isdir(basepath):
                basepath = os.path.join(CT_SOFTWARE_HOME, prodpath)
            
            # first, open file to see how long it is.
            name = name.replace('/', os.sep).replace('\\',os.sep)
            
            t = os.stat(os.path.join(basepath, name+ '.zpt'))
            mtime = t[stat.ST_MTIME]
            
            added = write2config(basepath, name, description,
                                 mtime, 'ZPT')
            
            if DEBUG and added:
                LOG(self.__class__.__name__, INFO, name)
            
        apply(PageTemplateFile.__init__, (self, name, _prefix), kw)

    def _exec(self, bound_data, args, kw):
        """ doc string """
        if len(self.ctnamesplit) == 1:
            possibletemplate = self.ctnamesplit[0]+'.zpt'
        else:
            possibletemplate = '.'.join(self.ctnamesplit[1:]) + '.zpt'
        base = self

        if hasattr(base, possibletemplate):
            request = self.REQUEST
            result = apply(getattr(base, possibletemplate), (self, request))
        else:
            result = apply(PageTemplateFile._exec, (self, bound_data, args, kw))
	
        if self.optimize and OPTIMIZE:
            result = slimmer(result, self.optimize)
	    
        return result
	

